#!/usr/bin/env python
from wxPython.wx import *
import xmlrpclib, urllib, urlparse

class ConnectionDialog(wxDialog):
    """
    Zope Connection Dialog
    """
    def __init__(self, parent, id, title):
        wxDialog.__init__(self, parent, -1, title, size=(300, 170))
        wxStaticText(self, -1, "URL:", (10, 10))
        self.url=wxTextCtrl(self, -1, "http://localhost:8080", (70, 10), (200, 20))
        wxStaticText(self, -1, "User name:", (10, 35))
        self.username=wxTextCtrl(self, -1, "superuser", (70, 35), (100, 20))
        wxStaticText(self, -1, "Password:", (10, 60))
        self.password=wxTextCtrl(self, -1, "123", (70, 60), (100, 20), wxTE_PASSWORD)
        wxButton(self, wxID_OK, " OK ", (50, 100)).SetDefault()
        wxButton(self, wxID_CANCEL, " Cancel ", (175, 100))

class ZopeListCtrl(wxListCtrl):
    def connectZope(self, url, username, password):
        zopeobj = xmlrpclib.Server(url, username=username, password=password)
        try:
            tmp = zopeobj.objectIds()
        except:
            pass
        else:
            parsedurl = urlparse.urlparse(url)
            self.host = parsedurl[1]
            self.path = parsedurl[2]
            self.username = username
            self.password = password
            self.zopeobj = zopeobj
            self.serverobj = zopeobj
            self.updateList()
        
    def updateList(self):
        wxSetCursor(wxStockCursor(wxCURSOR_WAIT))
        self.DeleteAllItems()        
        for id in self.zopeobj.objectIds():
            i = self.InsertItem(wxListItem())
            self.SetStringItem(i, 0, id)            
            try:
                subobj = getattr(self.zopeobj, id)                
                subobj.objectIds()
            except: 
                self.SetStringItem(i, 0, id, 1)
            else:
                self.SetStringItem(i, 0, id, 0)
                try: self.SetStringItem(i, 1, subobj.getProperty('title'))
                except: pass
            
            self.SetItemText(i, id)
        i = self.InsertItem(wxListItem())
        self.SetStringItem(i, 0, '..')
        for j in range(i):
            self.EnsureVisible(j)
        wxSetCursor(wxStockCursor(wxCURSOR_ARROW))
    
    def onItemSelected(self, event):
        self.currentItem = event.m_itemIndex
            
    def onDoubleClick(self, event):
        self.chdir(self.GetItemText(self.currentItem))
    
    def chdir(self, dirname):
        if dirname != '..':
            try: self.zopeobj = getattr(self.zopeobj, dirname)
            except: pass
        else:
            jn = string.join
            sp = string.split
            # truncate last item in path
            newpath = jn(sp(self.path, '/')[:-1], '/')
            self.path = newpath
            self.zopeobj = xmlrpclib.Server('http://%s/%s' % (self.host, self.path),
                                     username=self.username, password=self.password)
        self.updateList()


class MainFrame(wxFrame):
    def __init__(self, parent, ID, title):
        wxFrame.__init__(self, parent, ID, title,
                         wxDefaultPosition, wxSize(450, 400))
                         
        # --
        
        self.CreateStatusBar()
        self.SetStatusText("Welcome to LoadSite")
        
        splitter = wxSplitterWindow(self, -1)
        
        # make image list
        self.images=wxImageList(15, 16)
        wxImage_AddHandler(wxPNGHandler())
        self.images.Add(wxImage('folder.png', wxBITMAP_TYPE_PNG).ConvertToBitmap())
        self.images.Add(wxImage('doc.png', wxBITMAP_TYPE_PNG).ConvertToBitmap())
        
        # Make local file list
        lID = NewId()
        self.filelist = wxListCtrl(splitter, lID, wxDefaultPosition, wxDefaultSize,
                                   wxLC_REPORT)
        self.filelist.SetImageList(self.images, wxIMAGE_LIST_SMALL)
        self.filelist.InsertColumn(0, "Name")
        self.filelist.InsertColumn(1, "Type")
        
        # Make zope objects list
        lID = NewId()
        self.zopelist =  ZopeListCtrl(splitter, lID, wxDefaultPosition, wxDefaultSize,
                                   wxLC_REPORT)
        self.zopelist.SetImageList(self.images, wxIMAGE_LIST_SMALL)
        self.zopelist.InsertColumn(0, "Name")
        self.zopelist.InsertColumn(1, "Title")
        self.zopelist.parentwindow = self
        EVT_LEFT_DCLICK(self.zopelist, self.zopelist.onDoubleClick)
        EVT_LIST_ITEM_SELECTED(self.zopelist, lID, self.zopelist.onItemSelected)
        
        # Arrange windows
        splitter.SplitVertically(self.filelist, self.zopelist)
        splitter.SetSashPosition(180, true)
        splitter.SetMinimumPaneSize(20)
        
        # Make connection dialog
        self.connect=ConnectionDialog(self, -1, "Connect to Zope")
        
        # Make File menu
        menu = wxMenu()
        mID = NewId()
        menu.Append(mID, '&Connect', 'Connect to Zope')
        EVT_MENU(self, mID, self.onFileConnect)
        mID = NewId()
        menu.Append(mID, 'E&xit', 'Quit this program')
        EVT_MENU(self, mID, self.onFileExit)

        menuBar = wxMenuBar()
        menuBar.Append(menu, "&File");
        self.SetMenuBar(menuBar)

        #self.zopelist.connectZope("http://zope:8080", "superuser", "123")

       
    def onFileExit(self, event):
        self.Close(true)
        
    def onFileConnect(self, event):
        val = self.connect.ShowModal()
        if val == wxID_OK:
            username = self.connect.username.GetValue()
            password = self.connect.password.GetValue()
            url = self.connect.url.GetValue()
            self.zopelist.connectZope(url, username, password)
    
           
class LoadSiteApp(wxApp):
    def OnInit(self):
        frame = MainFrame(NULL, -1, "LoadSite")
        frame.Show(true)
        self.SetTopWindow(frame)
        return true

app = LoadSiteApp(0)
app.MainLoop()
